@@ -6,10 +6,15 @@
 
     sub new {
         my $class = shift;
-        my $args  = { ( @_ == 1 ) ? %{ $_[0] } : @_ };
-
-        my $self = bless {}, $class;
+        my $meta  = ( $Mite::META{$class} ||= $class->__META__ );
+        my $self  = bless {}, $class;
+        my $args =
+            $meta->{HAS_BUILDARGS}
+          ? $class->BUILDARGS(@_)
+          : { ( @_ == 1 ) ? %{ $_[0] } : @_ };
+        my $no_build = delete $args->{__no_BUILD__};
 
+        # Initialize attributes
         if ( exists( $args->{q[deck]} ) ) {
             (
                 do {
@@ -23,7 +28,6 @@
 q[Type check failed in constructor: deck should be InstanceOf["Acme::Mitey::Cards::Deck"]]
               );
             $self->{q[deck]} = $args->{q[deck]};
-            delete $args->{q[deck]};
         }
         require Scalar::Util && Scalar::Util::weaken( $self->{q[deck]} );
         if ( exists( $args->{q[reverse]} ) ) {
@@ -39,17 +43,50 @@ q[Type check failed in constructor: deck should be InstanceOf["Acme::Mitey::Card
               && Carp::croak(
                 q[Type check failed in constructor: reverse should be Str]);
             $self->{q[reverse]} = $args->{q[reverse]};
-            delete $args->{q[reverse]};
         }
 
-        keys %$args
+        # Enforce strict constructor
+        my @unknown = grep not(
+            do {
+
+                package Acme::Mitey::Cards::Mite;
+                ( defined and !ref and m{\A(?:(?:deck|reverse))\z} );
+            }
+          ),
+          keys %{$args};
+        @unknown
           and require Carp
-          and Carp::croak( "Unexpected keys in constructor: "
-              . join( q[, ], sort keys %$args ) );
+          and Carp::croak(
+            "Unexpected keys in constructor: " . join( q[, ], sort @unknown ) );
+
+        # Call BUILD methods
+        !$no_build and @{ $meta->{BUILD} || [] } and $self->BUILDALL($args);
 
         return $self;
     }
 
+    sub BUILDALL {
+        $_->(@_) for @{ $Mite::META{ ref( $_[0] ) }{BUILD} || [] };
+    }
+
+    sub __META__ {
+        no strict 'refs';
+        require mro;
+        my $class      = shift;
+        my $linear_isa = mro::get_linear_isa($class);
+        return {
+            BUILD => [
+                map { ( *{$_}{CODE} ) ? ( *{$_}{CODE} ) : () }
+                map { "$_\::BUILD" } reverse @$linear_isa
+            ],
+            DEMOLISH => [
+                map   { ( *{$_}{CODE} ) ? ( *{$_}{CODE} ) : () }
+                  map { "$_\::DEMOLISH" } reverse @$linear_isa
+            ],
+            HAS_BUILDARGS => $class->can('BUILDARGS'),
+        };
+    }
+
     my $__XS = !$ENV{MITE_PURE_PERL}
       && eval { require Class::XSAccessor; Class::XSAccessor->VERSION("1.19") };
 
@@ -65,7 +102,7 @@ q[Type check failed in constructor: deck should be InstanceOf["Acme::Mitey::Card
             @_ > 1
               ? require Carp
               && Carp::croak("deck is a read-only attribute of @{[ref $_[0]]}")
-              : $_[0]->{q[deck]};
+              : $_[0]{q[deck]};
         };
     }
 
